############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

from pyGlobus.gassCache import GassCache
from pyGlobus import gassCopy, gassCachec
from pyGlobus.util import GlobusException
import ConfigParser, unittest


#The config file is ../../test/config.py
config = None # ConfigParser object

SECTION = "gassCache"
URL = "url"
TAG = "tag"


#############################################################################
#  Unit test for the gassCache Module.
##############################################################################
class GassCacheTestCase(unittest.TestCase):
    "Test case for the GassCache module"

    def setUp(self):
        global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.py")
            print dir(config)
        print "\nGassCache Test Case begin\n"
        return

    def tearDown(self):
        print "\nGassCacheTestCase finished\n"
        return

    def check_cache_query(self):
        print 'begin check_cache_query'
        url = config.get(SECTION, URL)      # add(), and close()
        tag = config.get(SECTION, TAG)      # add(), and close()
        try:
            gCache = GassCache()
            gCache.open()
            ret, file,timestamp  = gCache.add(url, tag, 1)
            if ret == gassCachec.GLOBUS_GASS_CACHE_ADD_EXISTS:
                ## File already exists.  Mark it 'done' and continue.
                ## Note that because we 'cleanup_tag' at the end of
                ## the test, this code path will probably not be
                ## executed unless somebody else has left the cache populated with this entry                
                ret = gCache.add_done(url, tag,timestamp)
            elif ret == gassCachec.GLOBUS_GASS_CACHE_ADD_NEW:
                ## File wasn't in cache.  Add it.
                g = gassCopy.GassCopy(handleAttr=gassCopy.HandleAttr(), handle=None)
                try:
                    ## DEK: Should check for success here (ret != GLOBUS_SUCCESS). Delete the cache entry if the transfer failed.
                    ## NJE: Library layer functions shouldn't return GLOBUS_SUCCESS, but rather throw exceptions.
                    g.copy_url_to_url(srcUrl=url, srcAttr=gassCopy.Attr(), destUrl="file://" + file, destAttr=gassCopy.Attr())
                except Exception, ex:
                    ## NJE: Replaced SWIG layer command with API layer.
                    gCache.delete(url, tag, timestamp, 1)                    
                ret = gCache.add_done(url, tag,timestamp)
            elif ret == GLOBUS_SUCCESS or ret == None:
                ret = gCache.add_done(url, tag,timestamp)
            else:
                raise RuntimeError, "Hey: you were supposed to handle this case!"
            gCache.close()

            gCache = GassCache()
            gCache.open()
            incache = gCache.cache_query(url,tag,False,timestamp,file)
            gCache.cleanup_tag(url, tag)
            gCache.close()
            del gCache
        except GlobusException, ex:
            print "Exception occured during testing."
            assert 0, ex.args
        print 'end check_cache_query'
        return

    def check_cache_query_failure(self):
        print 'begin check_cache_query'
        url = config.get(SECTION, URL)      # add(), and close()
        tag = config.get(SECTION, TAG)      # add(), and close()

        try:
            gCache = GassCache()
            gCache.open()
            incache = gCache.cache_query(url,tag,False,0,"file")
            gCache.cleanup_tag(url, tag)
            gCache.close()
            del gCache
        except Exception, ex:
            assert ex.args[0] == 'Invalid error code'
        print 'end check_cache_query'
        return
    
    def check_cache_get_dirs(self):
        print 'in check_get_dirs'
        url = config.get(SECTION, URL)      
        tag = config.get(SECTION, TAG)      
        try:
            gCache = GassCache()
            gCache.open()
            ret = gCache.cache_get_dirs(url,tag)
            print ret
            del gCache
        except GlobusException, ex:
            print "Exception occured during testing."
            assert 0, ex.args
        print 'outof check_get_dirs'
        

    def check_mangle_tag(self):
        print 'in check_mangle_tag'
        tag = config.get(SECTION, TAG)      # mangle_tag
        try:
            gCache = GassCache()
            gCache.open()
            print 'tag: ', tag
            mangle = gCache.mangle_tag(tag)
            print 'mangle: ',mangle
            assert mangle!=None, "\n\nmangle_tag('%s') is returning None!"%tag

            del gCache
        except GlobusException, ex:
            print "Exception occured during testing.", ex
            assert 0, ex.args
        print 'outof check_mangle_tag'

    
    def check_mangle_url(self):
        print 'in check_mangle_url'
        url = config.get(SECTION, URL)      # mangle_url
        try:
            gCache = GassCache()
            gCache.open()
            print 'url: ', url
            print 'mangle: ',gCache.mangle_url(url)
            del gCache
        except GlobusException, ex:
            print "Exception occured during testing."
            assert 0, ex.args
        print 'outof check_mangle_url'
    
    def checkGassAddCleanupTagCache(self):  # tests add_done(), cleanup_tag(), open()
        print 'begin testing GassAddCleanupTagCache'
        url = config.get(SECTION, URL)      # add(), and close()
        try:
            gCache = GassCache()
            gCache.open()
            ret, file,timestamp  = gCache.add(url, "foobar", 1)
            gCache.add_done(url, "foobar",timestamp)
            gCache.cleanup_tag(url, "foobar")
            gCache.close()
            del gCache
        except GlobusException, ex:
            print "Exception occured during testing."
            assert 0, ex.args
        print 'end testing GassAddCleanupTagCache'
        return
    
    def checkGassAddDeleteCache(self):
        print 'begin testing GassAddDeleteTagCache'
        url = config.get(SECTION, URL)      # tests above, except with delete_start()
        try:                                # and delete().
            gCache = GassCache()
            gCache.open()
            ret, file,timestamp = gCache.add(url, "foobar", 1)
            gCache.add_done(url, "foobar",timestamp)
            timestamp = gCache.delete_start(url, "foobar")
            gCache.delete(url, "foobar", timestamp, 1)
            gCache.close()
            del gCache
        except GlobusException, ex:
            print "Exception occured during testing."
            assert 0, ex.args

    

            
def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(GassCacheTestCase, "check"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")


if __name__ == "__main__" : main()
    
